<?php

namespace Drupal\search_api_rabbit_hole\Plugin\search_api\processor;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\search_api\IndexInterface;
use Drupal\search_api\Processor\ProcessorPluginBase;

/**
 * Excludes some Rabbit Holed entities from being indexed.
 *
 * @SearchApiProcessor(
 *   id = "indexing_rabbit_hole_filter",
 *   label = @Translation("Indexing Rabbit Hole Filter"),
 *   description = @Translation("Prevent some items from being indexed depending on rabbit hole settings."),
 *   stages = {
 *     "alter_items" = 0,
 *   },
 * )
 */
class IndexingRabbitHoleFilter extends ProcessorPluginBase {

  /**
   * {@inheritdoc}
   */
  public static function supportsIndex(IndexInterface $index) {
    /** @var \Drupal\rabbit_hole\Plugin\RabbitHoleEntityPluginManager $rabbit_hole_entity_manager */
    $rabbit_hole_entity_manager = \Drupal::service('plugin.manager.rabbit_hole_entity_plugin');
    $rabbit_hole_entity_type_ids = $rabbit_hole_entity_manager->loadSupportedEntityTypes();

    foreach ($index->getDatasources() as $datasource) {
      $entity_type_id = $datasource->getEntityTypeId();
      if (!$entity_type_id) {
        continue;
      }

      if (in_array($entity_type_id, $rabbit_hole_entity_type_ids)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function alterIndexedItems(array &$items) {
    $no_index_rabbit_hole_plugins = [
      'access_denied',
      'page_not_found',
    ];
    foreach ($items as $item_id => $item) {

      // Check if the entity has Rabbit Hole settings stored.
      /** @var \Drupal\search_api\Item\ItemInterface $item */
      $entity = $item->getOriginalObject()->getValue();
      if (
        $entity instanceof ContentEntityInterface
        && $entity->hasField('rh_action')
      ) {

        /** @var \Drupal\rabbit_hole\BehaviorInvokerInterface $rabbit_hole_settings */
        /* @phpstan-ignore-next-line */
        $rabbit_hole_settings = \Drupal::service('rabbit_hole.behavior_invoker')->getRabbitHoleValuesForEntity($entity);
        if (in_array($rabbit_hole_settings['rh_action'], $no_index_rabbit_hole_plugins)) {
          unset($items[$item_id]);
        }
      }
    }
  }

}
